@@ -7,13 +7,15 @@ use integer;
 use B;
 use Carp ();
 use Time::Local qw/timelocal_nocheck timegm_nocheck/;
-use Encode qw/decode/;
+use Encode qw/decode is_utf8/;
 use Encode::Locale;
 use Locale::Scope qw/locale_scope/;
 use List::MoreUtils qw/uniq/;
 use POSIX qw/tzset strftime LC_ALL/;
 
-our $VERSION = 0.02_01;
+use constant DEBUG => exists $ENV{PERL_TIME_STRPTIME_DEBUG} && $ENV{PERL_TIME_STRPTIME_DEBUG};
+
+our $VERSION = 0.03;
 
 BEGIN {
     local $@;
@@ -24,9 +26,9 @@ BEGIN {
 our %DEFAULT_HANDLER = (
     '%' => [char          => '%' ],
     A   => [SKIP          => sub {
-        map quotemeta, map { ($_, substr $_, 0, 3) } map { decode(locale => $_) } map {
+        map quotemeta, map { ($_, substr $_, 0, 3) } map { is_utf8($_) ? $_ : decode(locale => $_) } map {
             strftime('%a', 0, 0, 0, $_, 0, 0),
-                strftime('%A', 0, 0, 0, $_, 0, 0)
+            strftime('%A', 0, 0, 0, $_, 0, 0)
         } 1..7
     }],
     a   => [extend        => q{%A} ],
@@ -36,8 +38,10 @@ our %DEFAULT_HANDLER = (
         unless (exists $self->{format_table}{localed_month}) {
             my %format_table;
             for my $month (1..12) {
-                $format_table{decode(locale => strftime('%b', 0, 0, 0, 1, $month-1, 0))} = $month;
-                $format_table{decode(locale => strftime('%B', 0, 0, 0, 1, $month-1, 0))} = $month;
+                my $b = strftime('%b', 0, 0, 0, 1, $month-1, 0);
+                my $B = strftime('%B', 0, 0, 0, 1, $month-1, 0);
+                $format_table{is_utf8($b) ? $b : decode(locale => $b) } = $month;
+                $format_table{is_utf8($B) ? $B : decode(locale => $B) } = $month;
             }
             $format_table{substr($_, 0, 3)} = $format_table{$_} for keys %format_table;
             $self->{format_table}{localed_month} = \%format_table;
@@ -158,7 +162,7 @@ sub _compile_format {
 my ($vars);
 \$offset = 0;
 sub {
-    ($captures) = \$_[0] =~ m{\\A$format\\z}mo
+    ($captures) = \$_[0] =~ m{^$format\$}
         or Carp::croak 'cannot parse datetime. text: "'.\$_[0].'", format: '.\%s;
 \%s
     (\$epoch, \$offset);
@@ -175,7 +179,7 @@ EOD
 
         my $combined_src = sprintf $parser_src, B::perlstring(B::perlstring($self->{format})), $formatter_src;
         $self->{parser_src} = $combined_src;
-        # warn $combined_src;
+        warn Encode::encode_utf8 "[DEBUG] src: $combined_src" if DEBUG;
 
         my $format_table = $self->{format_table} || {};
         eval $combined_src; ## no critic
@@ -204,13 +208,13 @@ sub _assemble_format {
             my $prefix = quotemeta($1||'');
             my $suffix = quotemeta($3||'');
             $prefix.$self->_assemble_format($2, $types).$suffix
-        }geo;
+        }ge;
         return $val;
     }
     else {
-        return ''   if $type eq 'TODO' and warn "SKIP(TODO): \%$c. patches welcome :)";
-        return $val if $type eq 'SKIP';
-        return $val if $type eq 'char';
+        return ''             if $type eq 'TODO' and warn "SKIP(TODO): \%$c. patches welcome :)";
+        return quotemeta $val if $type eq 'SKIP';
+        return quotemeta $val if $type eq 'char';
 
         push @$types => $type;
         return "($val)";
